package de.intevation.flys.artifacts.services.meta;

import java.io.InputStream;
import java.io.IOException;
import java.io.File;

import java.io.FileInputStream;

import org.apache.log4j.Logger;

import org.w3c.dom.Document;

import de.intevation.artifacts.common.utils.Config;
import de.intevation.artifacts.common.utils.XMLUtils;

public class DataCage
{
    private static Logger log = Logger.getLogger(DataCage.class);

    public static final String XPATH_META_DATA_TEMPLATE =
        "/artifact-database/metadata/@template";

    private static DataCage INSTANCE;

    protected Builder builder;

    public DataCage() {
    }

    public DataCage(Builder builder) {
        this.builder = builder;
    }

    public Builder getBuilder() {
        return builder;
    }

    public static synchronized DataCage getInstance() {
        if (INSTANCE == null) {
            INSTANCE = createDataCage();
        }
        return INSTANCE;
    }

    protected static Builder createBuilder(File file) {
        log.debug("DataCage.createBuilder");

        if (!file.isFile() || !file.canRead()) {
            log.error("Cannot open template file '" + file + "'");
            return null;
        }

        InputStream in = null;

        try {
            in = new FileInputStream(file);

            Document template = XMLUtils.parseDocument(in);

            if (template == null) {
                log.error("cannot parse meta data template");
            }
            else {
                return new Builder(template);
            }
        }
        catch (IOException ioe) {
            log.error(ioe);
        }
        finally {
            if (in != null) {
                try {
                    in.close();
                }
                catch (IOException ioe) {
                    log.error(ioe);
                }
            }
        }
        return null;
    }

    protected static DataCage createDataCage() {
        log.debug("DataCage.createDataCage");

        String path = Config.getStringXPath(XPATH_META_DATA_TEMPLATE);
        if (path == null) {
            log.error("no path to template file given");
            return null;
        }

        path = Config.replaceConfigDir(path);

        return new DataCage(createBuilder(new File(path)));
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf-8 :
